@@ -1,19 +1,8 @@
 import copy
-import operator
 import typing
+from contextlib import contextmanager
 from dataclasses import dataclass
-from pprint import pprint
-from typing import (
-    Any,
-    ClassVar,
-    Generic,
-    Literal,
-    Optional,
-    Protocol,
-    Type,
-    TypedDict,
-    TypeVar,
-)
+from typing import Any, ClassVar, Generic, Optional, Protocol, Type, TypeVar
 
 from fastapi import HTTPException
 from fastapi_resources.resources import base_resource, types
@@ -61,7 +50,7 @@ class SQLResourceProtocol(types.ResourceProtocol, Protocol, Generic[TDb]):
 
     engine: ClassVar[Optional[Engine]] = None
 
-    session: Session
+    session: Optional[Session]
 
     registry: dict[Type[SQLModel], type["BaseSQLResource"]] = {}
 
@@ -90,6 +79,9 @@ class SQLResourceProtocol(types.ResourceProtocol, Protocol, Generic[TDb]):
     def get_select(self) -> SelectOfScalar[SQLModel]:
         ...
 
+    def _get_session(self) -> Session:
+        ...
+
 
 def get_related_schema(annotation: Any):
     args = typing.get_args(annotation)
@@ -209,21 +201,30 @@ class BaseSQLResource(base_resource.Resource, SQLResourceProtocol[TDb], Generic[
 
     def __init__(
         self,
-        session: Session = None,
+        session: Optional[Session] = None,
         inclusions: Optional[types.Inclusions] = None,
         *args,
         **kwargs,
     ):
-        if session:
-            self.session = session
-        else:
+        # TODO: Warn about session management if a bound session is provided
+        self.session = session
+
+        if not session:
             assert self.engine, "A session or an engine must be given."
-            self.session = Session(self.engine)
 
         # TODO: Save the relationships on the instance at instantiation for caching
 
         super().__init__(inclusions=inclusions)
 
+    @contextmanager
+    def _get_session(self):
+        if self.session:
+            yield self.session
+            return
+
+        with Session(self.engine) as session:
+            yield session
+
     @classmethod
     def get_relationships(cls) -> Relationships:
         """Builds the relationship graph for the current resource.
@@ -283,10 +284,11 @@ class BaseSQLResource(base_resource.Resource, SQLResourceProtocol[TDb], Generic[
     ) -> SQLModel:
         select = self.get_select()
 
-        try:
-            return self.session.exec(select.where(self.Db.id == id)).unique().one()
-        except sa_exceptions.NoResultFound:
-            raise HTTPException(status_code=404, detail=f"{self.name} not found")
+        with self._get_session() as session:
+            try:
+                return session.exec(select.where(self.Db.id == id)).unique().one()
+            except sa_exceptions.NoResultFound:
+                raise HTTPException(status_code=404, detail=f"{self.name} not found")
 
     def get_related(
         self,
@@ -351,45 +353,49 @@ class CreateResourceMixin:
         for key, value in kwargs.items():
             setattr(row, key, value)
 
-        self.session.add(row)
-        self.session.commit()
+        with self._get_session() as session:
+            session.add(row)
+            session.commit()
 
-        model_relationships = self.get_relationships()
+            model_relationships = self.get_relationships()
 
-        if relationships:
-            save_row = False
+            if relationships:
+                save_row = False
 
-            for field, related_ids in relationships.items():
-                relationship = model_relationships[field]
-                direction = relationship.direction
+                for field, related_ids in relationships.items():
+                    relationship = model_relationships[field]
+                    direction = relationship.direction
 
-                if direction == ONETOMANY:
-                    assert isinstance(
-                        related_ids, list
-                    ), "A list of IDs must be provided for {field}"
+                    if direction == ONETOMANY:
+                        assert isinstance(
+                            related_ids, list
+                        ), "A list of IDs must be provided for {field}"
 
-                    related_resource = self.registry[
-                        relationship.schema_with_relationships.schema
-                    ]
-                    related_db_model = related_resource.Db
-                    new_related_ids = [rid for rid in related_ids]
+                        related_resource = self.registry[
+                            relationship.schema_with_relationships.schema
+                        ]
+                        related_db_model = related_resource.Db
+                        new_related_ids = [rid for rid in related_ids]
 
-                    # Update the related objects
-                    self.session.execute(
-                        update(related_db_model)
-                        .where(related_db_model.id.in_(new_related_ids))
-                        .values({relationship.update_field: row.id})
-                    )
+                        # Update the related objects
+                        session.execute(
+                            update(related_db_model)
+                            .where(related_db_model.id.in_(new_related_ids))
+                            .values({relationship.update_field: row.id})
+                        )
 
-                elif direction == MANYTOONE:
-                    # Can update locally via a setattr
-                    setattr(row, relationship.update_field, related_ids)
-                    save_row = True
+                    elif direction == MANYTOONE:
+                        # Can update locally via a setattr
+                        setattr(row, relationship.update_field, related_ids)
+                        save_row = True
 
-            if save_row:
-                self.session.commit()
+                if save_row:
+                    session.commit()
 
-            self.session.refresh(row)
+            # As the row is returned outside of the Session (i.e. it won't be bound
+            # to a session), we want to preselect all the relationships as determined
+            # by the `inclusions` provided when the resource was instantiated.
+            row = self.get_object(id=row.id)
 
         return row
 
@@ -410,47 +416,49 @@ class UpdateResourceMixin:
 
         model_relationships = self.get_relationships()
 
-        if relationships:
-            for field, related_ids in relationships.items():
-                relationship = model_relationships[field]
-                direction = relationship.direction
+        with self._get_session() as session:
+            if relationships:
+                for field, related_ids in relationships.items():
+                    relationship = model_relationships[field]
+                    direction = relationship.direction
 
-                if direction == ONETOMANY:
-                    assert isinstance(
-                        related_ids, list
-                    ), "A list of IDs must be provided for {field}"
+                    if direction == ONETOMANY:
+                        assert isinstance(
+                            related_ids, list
+                        ), "A list of IDs must be provided for {field}"
 
-                    related_resource = self.registry[
-                        relationship.schema_with_relationships.schema
-                    ]
-                    related_db_model = related_resource.Db
-                    new_related_ids = [rid for rid in related_ids]
+                        related_resource = self.registry[
+                            relationship.schema_with_relationships.schema
+                        ]
+                        related_db_model = related_resource.Db
+                        new_related_ids = [rid for rid in related_ids]
 
-                    # Update the related objects
-                    self.session.execute(
-                        update(related_db_model)
-                        .where(related_db_model.id.in_(new_related_ids))
-                        .values({relationship.update_field: id})
-                    )
-
-                    # Detach the old related objects
-                    # NOTE: This will raise if the foreign key is required. Is this OK?
-                    self.session.execute(
-                        update(related_db_model)
-                        .where(
-                            getattr(related_db_model, relationship.update_field) == id,
-                            related_db_model.id.not_in(new_related_ids),
+                        # Update the related objects
+                        session.execute(
+                            update(related_db_model)
+                            .where(related_db_model.id.in_(new_related_ids))
+                            .values({relationship.update_field: id})
                         )
-                        .values({relationship.update_field: None})
-                    )
 
-                elif direction == MANYTOONE:
-                    # Can update locally via a setattr
-                    setattr(row, relationship.update_field, related_ids)
+                        # Detach the old related objects
+                        # NOTE: This will raise if the foreign key is required. Is this OK?
+                        session.execute(
+                            update(related_db_model)
+                            .where(
+                                getattr(related_db_model, relationship.update_field)
+                                == id,
+                                related_db_model.id.not_in(new_related_ids),
+                            )
+                            .values({relationship.update_field: None})
+                        )
 
-        self.session.add(row)
-        self.session.commit()
-        self.session.refresh(row)
+                    elif direction == MANYTOONE:
+                        # Can update locally via a setattr
+                        setattr(row, relationship.update_field, related_ids)
+
+            session.add(row)
+            session.commit()
+            session.refresh(row)
 
         return row
 
@@ -459,7 +467,8 @@ class ListResourceMixin:
     def list(self: SQLResourceProtocol):
         select = self.get_select()
 
-        rows = self.session.exec(select).unique().all()
+        with self._get_session() as session:
+            rows = session.exec(select).unique().all()
 
         return rows
 
@@ -475,8 +484,9 @@ class DeleteResourceMixin:
     def delete(self: SQLResourceProtocol, *, id: int | str):
         row = self.get_object(id=id)
 
-        self.session.delete(row)
-        self.session.commit()
+        with self._get_session() as session:
+            session.delete(row)
+            session.commit()
 
         return {"ok": True}
 
