@@ -11,10 +11,9 @@ from typing import (
 )
 
 from fastapi import APIRouter, Request, Response
-from pydantic import BaseModel
-
 from fastapi_resources.resources.base_resource import Resource
 from fastapi_resources.routers import decorators
+from pydantic import BaseModel
 
 
 class TCreatePayload(BaseModel):
@@ -66,10 +65,16 @@ class ResourceRouter(APIRouter, Generic[TResource]):
         Read = self.resource_class.Read
         return List[Read]
 
+    def get_update_model(self):
+        return self.resource_class.Update
+
+    def get_create_model(self):
+        return self.resource_class.Create
+
     def get_method_replacements(self):
         return {
-            "_create": {TCreatePayload: self.resource_class.Create},
-            "_update": {TUpdatePayload: self.resource_class.Update},
+            "_create": {TCreatePayload: self.get_create_model()},
+            "_update": {TUpdatePayload: self.get_update_model()},
         }
 
     def _patch_route_types(self):
@@ -178,7 +183,10 @@ class ResourceRouter(APIRouter, Generic[TResource]):
                 )(func)
 
     def get_resource(self, request: Request):
-        return self.resource_class()
+        return self.resource_class(**self.get_resource_kwargs(request=request))
+
+    def get_resource_kwargs(self, request: Request):
+        return {}
 
     def build_response(
         self,
@@ -188,25 +196,49 @@ class ResourceRouter(APIRouter, Generic[TResource]):
     ):
         return rows
 
+    def parse_update(
+        self,
+        resource: TResource,
+        update: dict,
+    ):
+        resource_relationships = resource.get_relationships()
+
+        relationships = {}
+
+        for field in list(update.keys()):
+            value = update[field]
+            if field in resource_relationships:
+                relationships[field] = value
+                del update[field]
+
+        return update, relationships
+
     def perform_create(
-        self, request: Request, resource: TResource, create: TCreatePayload
+        self,
+        request: Request,
+        resource: TResource,
+        attributes: dict,
+        relationships: dict,
     ):
         if not resource.create:
             raise NotImplementedError("Resource.create not implemented")
 
-        return resource.create(model=create)
+        return resource.create(attributes=attributes, relationships=relationships)
 
     def perform_update(
         self,
         request: Request,
         resource: TResource,
         id: str | int,
-        update: TUpdatePayload,
+        attributes: dict,
+        relationships: dict,
     ):
         if not resource.update:
             raise NotImplementedError("Resource.update not implemented")
 
-        return resource.update(id=id, model=update)
+        return resource.update(
+            id=id, attributes=attributes, relationships=relationships
+        )
 
     def perform_delete(self, request: Request, resource: TResource, id: str | int):
         if not resource.delete:
@@ -233,15 +265,31 @@ class ResourceRouter(APIRouter, Generic[TResource]):
     def _create(self, *, create: TCreatePayload, request: Request):
         resource = self.get_resource(request=request)
 
-        row = self.perform_create(request=request, resource=resource, create=create)
+        attributes, relationships = self.parse_update(
+            resource=resource, update=create.dict(exclude_unset=True)
+        )
+
+        row = self.perform_create(
+            request=request,
+            resource=resource,
+            attributes=attributes,
+            relationships=relationships,
+        )
 
         return self.build_response(rows=row, resource=resource, request=request)
 
     def _update(self, *, id: Union[int, str], update: TUpdatePayload, request: Request):
         resource = self.get_resource(request=request)
 
+        attributes, relationships = self.parse_update(
+            resource=resource, update=update.dict(exclude_unset=True)
+        )
         row = self.perform_update(
-            request=request, resource=resource, update=update, id=id
+            request=request,
+            resource=resource,
+            attributes=attributes,
+            relationships=relationships,
+            id=id,
         )
 
         return self.build_response(rows=row, resource=resource, request=request)
